%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Fionnuala Connolly                                                  
%%% July 20th 2016                                                      
%%% In section 1, the user inputs the required bend angles, twist angles and link
%%% lengths which the actuator should achieve. The user also prescribes the
%%% segment types.
%%% In section 2, all parameters for the models are defined.
%%% In section 3, the optimization problem is solved.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all;close all;clc

%% Section 1: Input data
% Data should be in the form of mxn matrices, where m is the number of joints (for bend angles)
% or links (for link lengths) and n is the number of configurations.
% psi is the bend angles, th the twist angles, and L the link lengths

% Segment types: 0 means extending, 1 means bending, 2 means twisting
% segTypes = [0 1 0 1 0 1 0]; %finger (alternating extending and bending segments)
segTypes = [2 1 0 1 0 1 0]; %thumb (a twisting segment followed by alternating bending and extending segments)

%finger
% psi =[14.6694   27.5051   46.7613   72.2250
%    13.5758   40.3575   66.9560   92.7809
%     6.9478   19.0584   34.7656   53.8419];
% 
% th = [];
% 
% L = [81.2959   82.9036   84.2476   85.2898
%    46.9017   49.7775   52.8091   55.9350
%    25.8070   27.1771   29.5357   32.8633
%    23.2125   23.9417   24.6577   25.3444];

%thumb
th = [-1.0785   38.6160   64.2082   72.8807];

L = [42.7200   46.5609   50.2579   53.4876
   34.2520   39.2200   42.2819   43.0909
   27.6080   29.3539   29.9976   29.4355];


psi = [-2.1550    8.8650   21.2042   33.8622
   17.5992   38.3712   52.7630   59.2584
    5.8643   20.4960   33.7867   44.5371];

%% Section 2: Define parameters

% Choose a maximum pressure (in MPa):
Pmax=0.08;

% Choose max and min bend/twist segment lengths (in mm):
Lmin=10;
Lmax=30;

% Choose max and min fiber angles (in radians):
alphaMin=5*pi/180;
alphaMax=50*pi/180;

% actuator geometry:
Ri = 6.35; %actuator inner radius
Ro = Ri+2; %actuator outer radius
Rm = Ro-0.01*0.0889; %boundary between inner and outer layers
area = pi*0.0889*0.0889; %fiber cross sectional area (in mm^2)

% material parameters:
e = 31067; %fiber Young's modulus (in MPa)
mu1=0.085; %material 1 shear modulus (in MPa)
mu2=0.68; %material 2 shear modulus (in MPa)
muBar=0.078; %effective shear modulus for bending model (in MPa)

% fitting parameter for bend model:
phi=35*pi/180; %this defines where the neutral bending axis is

%% Section 3: Solve the optimization problem
% Solve for (1) the fiber angles, (2) the initial lengths of the
% bending and twisting segments (3) the pressures at which the required
% configurations are achieved

numSteps=size(psi,2); %number of configurations to match
numLinks = size(L,1); %number of links
numSegsInit=length(segTypes); %number of segments
numBendSegsInit=sum(segTypes==1); %number of bend segments
numTwistSegsInit=sum(segTypes==2);

%objective function:
fun=@(x)(bend_extend_twist_function(x,psi,th,L,numSegsInit,numBendSegsInit,numSteps,Ri,Rm,Ro,e,area,mu1,mu2,muBar,phi,segTypes));

%min and max values of variables:
xMin = [zeros(1,numSteps),alphaMin*ones(1,numSegsInit),Lmin*ones(1,numBendSegsInit+numTwistSegsInit)];
xMax = [Pmax*ones(1,numSteps),alphaMax*ones(1,numSegsInit),Lmax*ones(1,numBendSegsInit+numTwistSegsInit)];

%initial values of variables:
xInit = [linspace(0,Pmax,numSteps) alphaMin*ones(1,numSegsInit) Lmin*ones(1,numBendSegsInit+numTwistSegsInit)];

%solve the problem:
options=optimset('display','iter');
tic
[solution,RESNORM,RESIDUAL]=lsqnonlin(fun,xInit,xMin,xMax,options);
toc

pressure = solution(1:numSteps);
alpha = solution(numSteps+1:numSteps+numSegsInit);
segLengths_init = calc_initial_lengths(solution(end-numBendSegsInit-numTwistSegsInit+1:end-numBendSegsInit),solution(end-numBendSegsInit+1:end),L,segTypes);

%% Print output

for i=1:length(segTypes)
    if segTypes(i)==0
        fprintf(['Segment %d: an extending segment of length %.2f','mm, with fiber angle %.2f%s. \n'],i,segLengths_init(i),alpha(i)*180/pi,char(176))
    elseif segTypes(i)==1
        fprintf(['Segment %d: a bending segment of length %.2f','mm, with fiber angle %.2f%s. \n'],i,segLengths_init(i),alpha(i)*180/pi,char(176))
    elseif segTypes(i)==2
        fprintf(['Segment %d: a twisting segment of length %.2f','mm, with fiber angle %.2f%s. \n'],i,segLengths_init(i),alpha(i)*180/pi,char(176))
    end
end

for i=1:numSteps
    fprintf(['Configuration %d occurs at P=%.2f kPa.\n'],i,pressure(i)*1000)
end

